/* ../src/voronoi.f -- translated by f2c (version 19950110).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

#include "f2c.h"

/* Table of constant values */

static integer c__3 = 3;

/* Subroutine */ int voronoi_(ncc, lcc, n, x, y, list, lptr, lend, nt, lccc, 
	iccc, lct, ltri, ier)
integer *ncc, *lcc, *n;
doublereal *x, *y;
integer *list, *lptr, *lend, *nt;
doublereal *lccc;
integer *iccc, *lct, *ltri, *ier;
{
    /* System generated locals */
    integer i__1;

    /* Local variables */
    static integer ntri, nrow;
    static logical ratio;
    extern /* Subroutine */ int qsort_();
    static doublereal x1, x2, x3, y1, y2, y3, ar, sa;
    static real cr;
    static integer ip[3];
    static doublereal xc, yc;
    static integer it;
    static doublereal xp[3], yp[3];
    extern /* Subroutine */ int circum_(), rmshnb_();
    static integer it1, it2;
    extern /* Subroutine */ int trlist_();


/*    DETERMINE A VORONOI MOSAIC. TO BE USED WITH THE FUNCTIONS FROM TRIPA
CK.*/

/*     THIS IS DONE BY DETERMINING THE CIRCUMCIRCLE CENTERS OF A */
/*    DELAUNAY TRIANGULATION, USING ITS TRIANGLE NEIGBOURSHIP RELATIONS AS
*/
/*     ADJACENCY RELATION FOR THE CIRCUMCIRCLE CENTERS. */

/*     THE TRIANGULATION PRODUCED WITH TRLIST FROM TRIPACK CAN */
/*     INCLUDE EMPTY TRIANGLES WITH AREA 0 (IF COLLINEAR POINTS EXIST). */
/*     THESE TRIANGLES MAKE TROUBLES IF THEY OCCUR ON THE BORDER OF THE */
/*     TRIANGULATION. SO WE MUST REMOVE THEM. */

/*     A.GEBHARDT */

/*     ON ENTRY: */
/*     ... SEE E.G. TRLIST */

/*     ON EXIT: */
/*     NT   - NUMBER OF TRIANGLES */
/*     LCCC - LIST OF CIRCUMCIRCLE CENTERS (3 by NT): */
/*            ROW 1,2: X, Y COORDINATES */
/*            ROW 3  : TRIANGLE AREA (-1 = REMOVED TRIANGLE) */
/*     ICCC   LIST OF NEIGHBOUR RELATIONS (6 by NT): */
/*            ROW 1-3: NEIGHBOUR TRIANGLE INDICES */
/*            ROW 4-6: NODE INDICES OF TRIANGLE */

/*     Modules required by VORONOI:  TRLIST, CIRCUM, QSORT, RMSHNB */
/*     DETERMINE ALL CIRCUMCIRCLE CENTERS */
    /* Parameter adjustments */
    --lct;
    --lcc;
    --lend;
    --x;
    --y;
    --list;
    --lptr;
    lccc -= 4;
    iccc -= 7;
    ltri -= 10;

    /* Function Body */
    ratio = FALSE_;
    nrow = 9;
    trlist_(ncc, &lcc[1], n, &list[1], &lptr[1], &lend[1], &nrow, nt, &ltri[
	    10], &lct[1], ier);
    ntri = *nt;
    i__1 = ntri;
    for (it = 1; it <= i__1; ++it) {
	x1 = x[ltri[it * 9 + 1]];
	y1 = y[ltri[it * 9 + 1]];
	x2 = x[ltri[it * 9 + 2]];
	y2 = y[ltri[it * 9 + 2]];
	x3 = x[ltri[it * 9 + 3]];
	y3 = y[ltri[it * 9 + 3]];
	circum_(&x1, &y1, &x2, &y2, &x3, &y3, &ratio, &xc, &yc, &cr, &sa, &ar)
		;
	lccc[it * 3 + 1] = xc;
	lccc[it * 3 + 2] = yc;
	lccc[it * 3 + 3] = sa;
	iccc[it * 6 + 1] = ltri[it * 9 + 4];
	iccc[it * 6 + 2] = ltri[it * 9 + 5];
	iccc[it * 6 + 3] = ltri[it * 9 + 6];
	iccc[it * 6 + 4] = ltri[it * 9 + 1];
	iccc[it * 6 + 5] = ltri[it * 9 + 2];
	iccc[it * 6 + 6] = ltri[it * 9 + 3];
/* L10: */
    }
/*    REMOVE TRIANGLES WITH AREA 0 FROM THE BORDER, UPDATE NEIGBOUR RELATI
ONS.*/
/*    AREA=0 MEANS COLLINEAR POINTS, 3 CASES (I'M NOT SURE IF THEY ALL WIL
L*/
/*     OCCUR!): */
/*       ALL POINTS DIFFERENT */
/*       2   POINTS COINCIDE */
/*       ALL POINTS COINCIDE */

L11:
    i__1 = ntri;
    for (it = 1; it <= i__1; ++it) {
	if (lccc[it * 3 + 3] == 0. && (iccc[it * 6 + 1] == 0 || iccc[it * 6 + 
		2] == 0 || iccc[it * 6 + 3] == 0)) {
/*     WE HAVE A 0-TRIANGLE AT THE BORDER */
	    xp[0] = x[iccc[it * 6 + 4]];
	    xp[1] = x[iccc[it * 6 + 5]];
	    xp[2] = x[iccc[it * 6 + 6]];
	    yp[0] = y[iccc[it * 6 + 4]];
	    yp[1] = y[iccc[it * 6 + 5]];
	    yp[2] = y[iccc[it * 6 + 6]];
	    qsort_(&c__3, xp, ip);
/*     CHECK IF ALL X AND/OR Y COORDINATES COINCIDE */
	    if (xp[ip[0] - 1] == xp[ip[2] - 1]) {
		qsort_(&c__3, xp, ip);
		if (yp[ip[0] - 1] == yp[ip[2] - 1]) {
/*     ALL POINTS COINCIDE, ALL NEIGBOUR TRIANGLES HAVE AR
EA 0 */
/*     AND WILL BE REMOVED LATER. */
/*     NOTHING TO DO (?) */
		} else {
/*     X1=X2=X3, USE Y-COORDINTES */
		    if (yp[ip[0] - 1] == yp[ip[1] - 1] || yp[ip[1] - 1] == yp[
			    ip[2] - 1]) {
/*     TWO POINTS COINCIDE */
			if (yp[ip[0] - 1] == yp[ip[1] - 1]) {
			    it1 = iccc[ip[0] + it * 6];
			    it2 = iccc[ip[1] + it * 6];
			    rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
			} else {
			    it1 = iccc[ip[1] + it * 6];
			    it2 = iccc[ip[2] + it * 6];
			    rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
			}
		    } else {
/*     ALL POINTS DIFFERENT */
			it1 = iccc[ip[0] + it * 6];
			it2 = iccc[ip[1] + it * 6];
			rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
			it1 = iccc[ip[2] + it * 6];
			it2 = iccc[ip[1] + it * 6];
			rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
		    }
		}
	    } else {
		if (xp[ip[0] - 1] == xp[ip[1] - 1] || xp[ip[1] - 1] == xp[ip[
			2] - 1]) {
/*     TWO POINTS COINCIDE */
		    if (xp[ip[0] - 1] == xp[ip[1] - 1]) {
			it1 = iccc[ip[0] + it * 6];
			it2 = iccc[ip[1] + it * 6];
			rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
		    } else {
			it1 = iccc[ip[1] + it * 6];
			it2 = iccc[ip[2] + it * 6];
			rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
		    }
		} else {
/*     ALL POINTS DIFFERENT */
		    it1 = iccc[ip[0] + it * 6];
		    it2 = iccc[ip[1] + it * 6];
		    rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
		    it1 = iccc[ip[1] + it * 6];
		    it2 = iccc[ip[2] + it * 6];
		    rmshnb_(&it, &it1, &it2, &lccc[4], &iccc[7]);
		}
	    }
	}
/* L20: */
    }
/*     REPEAT THE ABOVE STEPS UNTIL NO MORE AREA 0 TRIANGLES */
/*     OARE LEFT ON THE BORDER. */
    i__1 = ntri;
    for (it = 1; it <= i__1; ++it) {
	if (lccc[it * 3 + 3] == 0. && (iccc[it * 6 + 1] == 0 || iccc[it * 6 + 
		2] == 0 || iccc[it * 6 + 3] == 0)) {
	    goto L11;
	}
/* L30: */
    }
    return 0;
} /* voronoi_ */

/* Subroutine */ int rmshnb_(i0, i1, i2, lccc, iccc)
integer *i0, *i1, *i2;
doublereal *lccc;
integer *iccc;
{
    static integer k;

/*     REMOVE A SHARED NEIGBOUR TRIANGLE I0 FROM THE NEIGBOUR LISTS */
/*    OF TRIANGLE I1 AND I2 (REPLACE OCCURENCE OF I0 IN LIST OF I1 WITH I2
*/
/*     AND VICE VERSA.) */
    /* Parameter adjustments */
    iccc -= 7;
    lccc -= 4;

    /* Function Body */
    for (k = 1; k <= 3; ++k) {
	if (*i1 > 0) {
	    if (iccc[k + *i1 * 6] == *i0) {
		iccc[k + *i1 * 6] = *i2;
	    }
	}
	if (*i2 > 0) {
	    if (iccc[k + *i2 * 6] == *i0) {
		iccc[k + *i2 * 6] = *i1;
	    }
	}
/* L1: */
    }
/*     MARK TRIANGLE I0 AS REMOVED (AREA= -1) */
    lccc[*i0 * 3 + 3] = -1.;
    return 0;
} /* rmshnb_ */

